        PAGE    60,132
name    MSFIX
        TITLE   MSFIX Version 1.3
comment 

=================

MSFIX.COM

Purpose
*******

MSFIX.COM fixes the "packed file corrupt" problem when programs
such as MS Mouse install, Flight Simulator, Video Seven
Diagnostics, and PCTools load.  The problem shows up only in
some BIOSes.  The problem is a bug in exepack, a Microsoft
utility used to compress commercial software.  MSFIX is similar
to LOADFIX that comes bundled with DOS 5.0 and 6.0.

How It Works
************

The problem only shows up when the application programs load
starting in first 64K of RAM.  This will happen if you have only
a few TSRs, or if your DOS is stripped down.  MSFIX works by
allocating just enough RAM to force DOS to load subsequent
programs above the 64K boundary.

The MCB and environment for subsequently loaded programs will
also be allocated above the 64K boundary.

Notes on Use
************

It does not hurt to run MSFIX more than once.  If the the load
point is already above 64K, MSFIX does nothing.

NOTE: Do NOT use LOADHI from QEMM or 386MAX to load MSFIX into
high RAM.  If you do that it will have no effect -- harmful or
beneficial.

To avoid wasting RAM unnecessarily, load all your other TSRs first.

Author
******

Please report bugs and problems to:

Roedy Green
Canadian Mind Products
#208 - 525 Ninth Street
New Westminster BC Canada
V5H 2N6
tel:(604) 777-1804
mailto:roedy@mindprod.com
http://mindprod.com

Version 1.3 1998 November 8
- embed Barker address

Version 1.2 1996 October 25
- embed POB 707 Quathiaski Cove address

Version 1.1 1993 June 8
 - embed the new address and phone
 - more hints on how to fix via unexepacking

Version 1.0 1990 November 5
 - posted on BIX


;==============================================================

;       C O N V E N T I O N S

Subroutines may trash any registers they please except CS: DS:
ES: SS: and of course the outputs.

Some of the routines save more registers than this convention
requires.  This is because they were canibalized from other
environments where there were stricter rules.  Speed is not an
issue here.

 ; end of comment


;==============================================================

; M A C R O S

CR      MACRO   ; Carriage return line feed
        DB 0dh,0ah
        ENDM

;======

EOS     MACRO   ; marks end of display string
        DB 0dh,0ah,'$'
        ENDM

;=============================================================
stack   segment stack           ; keep MS link happy by providing null stack
stack   ends

CODE    SEGMENT PARA            ; start off in code.

data    segment byte            ; provide a separate DATA segment
                                ; actually all come after the code

;======================================
;
;       D I S P L A Y   S T R I N G S

CopyrightMsg    label byte
        DB      ' MSFIX 1.3 ۲'
        CR
        DB      'Fixes the Microsoft "packed file corrupt" bug.'
        CR
        DB      'Copyright (c) 1990,1998 Roedy Green Canadian Mind Products'
        CR
        DB      '#208 - 525 Ninth Street, New Westminster, BC Canada V3M 5T9',13,10
        DB      'tel:(604) 777-1804   mailto:roedy@mindprod.com   http://mindprod.com',13,10
        DB      'May be freely distributed and used for any purpose except military.'
        EOS
data            ends

;       P R O C E D U R E S

com     group   code,data       ; force data segment to go at the end!

        ASSUME  CS:com,DS:com,ES:com,SS:com

                                ; seg regs cover everything
        ORG     100H            ; in Code segment
;==============================================================

start:

Main    Proc    near
;       Initialize, then, possibly, Terminate and Stay Resident
        lea     dx,CopyRightMsg ; CopyRight banner
        call    SayCent         ; display centered
        mov     ax,CS
        cmp     ax,01000h       ; already above the 64K boundary?
        jae     AlreadyHigh     ; yes, jump
        call    FreeAll         ; free env, handles etc.
        mov     dx,01000h       ; 64 boundary (1/16 of address space)
        mov     ax,CS           ; ax is where we are now
        sub     dx,ax           ; dx # paras to get us to boundary
                                ; i.e. dx has paras to keep resident
        mov     ax,3100h        ; Terminate and Stay Resident
                                ; Errorlevel =0
                                ; Die
        int     21h

AlreadyHigh:                    ; just exit
        mov     ax, 4c00h       ; ERRORLEVEL = 0
        int     21h             ; normal exit to dos
Main    EndP

;==============================================================

FreeAll Proc    near
;       Free ENV and handles prior to Terminate and Stay Resident
        mov     ES,DS:[2Ch]     ; pointer to environment
        mov     ah,49h          ; free RAM allocated to SET environment
        int     21h

        MOV     AH,3Eh          ; Close StdIn
        MOV     BX,0            ; Avoid holding onto a handle if
        INT     21H             ; foolish user typed MSFIX <Myfile

        MOV     AH,3Eh          ; Close StdOut
        MOV     BX,1            ; Avoid holding onto a handle if
        INT     21H             ; user typed MSFIX >NUL:
        ret

FreeAll EndP

;============================================

Say     Proc    near
;       display message on screen
;       on entry DX contains address of $ delimited message.
        mov     AH,09h
        int     21h
        ret
Say     EndP

;============================================

SayCent Proc
;       on entry dx points to a string to display
;       it may contain CrLfs in it.  It is terminated by $.
;       We display each line centred on a 80 column screen.
LineLoop:
        mov     si,dx   ; si points to first char
        mov     di,dx   ; di save secord copy
        sub     bx,bx   ; count chars in line, 0 to start
                        ; si points to first char of line
CharLoop:
        lodsb           ; get first char
        cmp     al,'$'
        je      FoundEnd
        cmp     al,13d
        je      FoundEol
        inc     bx      ; found an ordinary char, count it.
        JMP     CharLoop
FoundEol:
                        ; found Cr, print centred line.
        mov     cx,80d  ; calc # spaces needed for centering
        sub     cx,bx   ; bx is count of real chars in line
        shr     cx,1    ; divide by 2
        and     cx,3Fh  ; just in case line too long.
        jcxz    SpacesDone
                        ; cx now contains number of spaces we want to emit.
SpaceLoop:
        mov     DL,' '
        mov     ah,02
        int     21H     ; emit enough spaces to centre the line.
        LOOP    SpaceLoop
SpacesDone:
        mov     cx,bx   ; bx contains length of string
        add     cx,2    ; account for CrLf at the end
        mov     dx,di   ; cx=count dx=addr
        mov     bx,1    ; handle 1 - standard output
        mov     ax,4000H ; ah=40 signifies write by handle
        int     21H     ; Write the string itself
        inc     si      ; point just past the LF
        mov     dx,si   ; point to start of next string.
        JMP     LineLoop; process the next line of text
FoundEnd:
                        ; We are done.  All lines have CrLf before $
        ret             ; so we need not process any tail end.
SayCent EndP
;============================================


CODE ends
end start
